@@ -16,15 +16,16 @@
 """Image V2 Action Implementations"""
 
 import argparse
-import six
+import logging
 
 from glanceclient.common import utils as gc_utils
+from osc_lib.cli import parseractions
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
+import six
 
 from openstackclient.api import utils as api_utils
-from openstackclient.common import command
-from openstackclient.common import exceptions
-from openstackclient.common import parseractions
-from openstackclient.common import utils
 from openstackclient.i18n import _
 from openstackclient.identity import common
 
@@ -33,8 +34,11 @@ DEFAULT_CONTAINER_FORMAT = 'bare'
 DEFAULT_DISK_FORMAT = 'raw'
 
 
+LOG = logging.getLogger(__name__)
+
+
 def _format_image(image):
-    """Format an image to make it more consistent with OSC operations. """
+    """Format an image to make it more consistent with OSC operations."""
 
     info = {}
     properties = {}
@@ -65,7 +69,7 @@ def _format_image(image):
 
 
 class AddProjectToImage(command.ShowOne):
-    """Associate project with image"""
+    _description = _("Associate project with image")
 
     def get_parser(self, prog_name):
         parser = super(AddProjectToImage, self).get_parser(prog_name)
@@ -103,7 +107,7 @@ class AddProjectToImage(command.ShowOne):
 
 
 class CreateImage(command.ShowOne):
-    """Create/upload an image"""
+    _description = _("Create/upload an image")
 
     deadopts = ('size', 'location', 'copy-from', 'checksum', 'store')
 
@@ -280,10 +284,8 @@ class CreateImage(command.ShowOne):
         project_arg = parsed_args.project
         if parsed_args.owner:
             project_arg = parsed_args.owner
-            self.log.warning(_(
-                'The --owner option is deprecated, '
-                'please use --project instead.'
-            ))
+            LOG.warning(_('The --owner option is deprecated, '
+                          'please use --project instead.'))
         if project_arg:
             kwargs['owner'] = common.find_project(
                 identity_client,
@@ -301,7 +303,7 @@ class CreateImage(command.ShowOne):
                                             "the same time"))
 
         if fp is None and parsed_args.file:
-            self.log.warning(_("Failed to get an image file."))
+            LOG.warning(_("Failed to get an image file."))
             return {}, {}
 
         if parsed_args.owner:
@@ -337,7 +339,7 @@ class CreateImage(command.ShowOne):
             with fp:
                 try:
                     image_client.images.upload(image.id, fp)
-                except Exception as e:
+                except Exception:
                     # If the upload fails for some reason attempt to remove the
                     # dangling queued image made by the create() call above but
                     # only if the user did not specify an id which indicates
@@ -347,7 +349,7 @@ class CreateImage(command.ShowOne):
                             image_client.images.delete(image.id)
                     except Exception:
                         pass  # we don't care about this one
-                    raise e  # now, throw the upload exception again
+                    raise  # now, throw the upload exception again
 
                 # update the image after the data has been uploaded
                 image = image_client.images.get(image.id)
@@ -359,7 +361,7 @@ class CreateImage(command.ShowOne):
 
 
 class DeleteImage(command.Command):
-    """Delete image(s)"""
+    _description = _("Delete image(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteImage, self).get_parser(prog_name)
@@ -372,17 +374,31 @@ class DeleteImage(command.Command):
         return parser
 
     def take_action(self, parsed_args):
+
+        del_result = 0
         image_client = self.app.client_manager.image
         for image in parsed_args.images:
-            image_obj = utils.find_resource(
-                image_client.images,
-                image,
-            )
-            image_client.images.delete(image_obj.id)
+            try:
+                image_obj = utils.find_resource(
+                    image_client.images,
+                    image,
+                )
+                image_client.images.delete(image_obj.id)
+            except Exception as e:
+                del_result += 1
+                LOG.error(_("Failed to delete image with name or "
+                            "ID '%(image)s': %(e)s"),
+                          {'image': image, 'e': e})
+
+        total = len(parsed_args.images)
+        if (del_result > 0):
+            msg = (_("Failed to delete %(dresult)s of %(total)s images.")
+                   % {'dresult': del_result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListImage(command.Lister):
-    """List available images"""
+    _description = _("List available images")
 
     def get_parser(self, prog_name):
         parser = super(ListImage, self).get_parser(prog_name)
@@ -474,6 +490,7 @@ class ListImage(command.Lister):
                 'Disk Format',
                 'Container Format',
                 'Size',
+                'Checksum',
                 'Status',
                 'visibility',
                 'protected',
@@ -486,6 +503,7 @@ class ListImage(command.Lister):
                 'Disk Format',
                 'Container Format',
                 'Size',
+                'Checksum',
                 'Status',
                 'Visibility',
                 'Protected',
@@ -524,7 +542,7 @@ class ListImage(command.Lister):
 
 
 class RemoveProjectImage(command.Command):
-    """Disassociate project with image"""
+    _description = _("Disassociate project with image")
 
     def get_parser(self, prog_name):
         parser = super(RemoveProjectImage, self).get_parser(prog_name)
@@ -557,7 +575,7 @@ class RemoveProjectImage(command.Command):
 
 
 class SaveImage(command.Command):
-    """Save an image locally"""
+    _description = _("Save an image locally")
 
     def get_parser(self, prog_name):
         parser = super(SaveImage, self).get_parser(prog_name)
@@ -585,7 +603,7 @@ class SaveImage(command.Command):
 
 
 class SetImage(command.Command):
-    """Set image properties"""
+    _description = _("Set image properties")
 
     deadopts = ('visibility',)
 
@@ -792,10 +810,8 @@ class SetImage(command.Command):
         project_arg = parsed_args.project
         if parsed_args.owner:
             project_arg = parsed_args.owner
-            self.log.warning(_(
-                'The --owner option is deprecated, '
-                'please use --project instead.'
-            ))
+            LOG.warning(_('The --owner option is deprecated, '
+                          'please use --project instead.'))
         if project_arg:
             kwargs['owner'] = common.find_project(
                 identity_client,
@@ -803,11 +819,6 @@ class SetImage(command.Command):
                 parsed_args.project_domain,
             ).id
 
-        # Checks if anything that requires getting the image
-        if not (kwargs or parsed_args.deactivate or parsed_args.activate):
-            self.log.warning(_("No arguments specified"))
-            return {}, {}
-
         image = utils.find_resource(
             image_client.images, parsed_args.image)
 
@@ -819,24 +830,21 @@ class SetImage(command.Command):
             image_client.images.reactivate(image.id)
             activation_status = "activated"
 
-        # Check if need to do the actual update
-        if not kwargs:
-            return {}, {}
-
         if parsed_args.tags:
             # Tags should be extended, but duplicates removed
             kwargs['tags'] = list(set(image.tags).union(set(parsed_args.tags)))
 
         try:
             image = image_client.images.update(image.id, **kwargs)
-        except Exception as e:
+        except Exception:
             if activation_status is not None:
-                print("Image %s was %s." % (image.id, activation_status))
-            raise e
+                LOG.info(_("Image %(id)s was %(status)s."),
+                         {'id': image.id, 'status': activation_status})
+            raise
 
 
 class ShowImage(command.ShowOne):
-    """Display image details"""
+    _description = _("Display image details")
 
     def get_parser(self, prog_name):
         parser = super(ShowImage, self).get_parser(prog_name)
@@ -856,3 +864,84 @@ class ShowImage(command.ShowOne):
 
         info = _format_image(image)
         return zip(*sorted(six.iteritems(info)))
+
+
+class UnsetImage(command.Command):
+    _description = _("Unset image tags and properties")
+
+    def get_parser(self, prog_name):
+        parser = super(UnsetImage, self).get_parser(prog_name)
+        parser.add_argument(
+            "image",
+            metavar="<image>",
+            help=_("Image to modify (name or ID)"),
+        )
+        parser.add_argument(
+            "--tag",
+            dest="tags",
+            metavar="<tag>",
+            default=[],
+            action='append',
+            help=_("Unset a tag on this image "
+                   "(repeat option to set multiple tags)"),
+        )
+        parser.add_argument(
+            "--property",
+            dest="properties",
+            metavar="<property_key>",
+            default=[],
+            action='append',
+            help=_("Unset a property on this image "
+                   "(repeat option to set multiple properties)"),
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        image_client = self.app.client_manager.image
+        image = utils.find_resource(
+            image_client.images,
+            parsed_args.image,
+        )
+
+        kwargs = {}
+        tagret = 0
+        propret = 0
+        if parsed_args.tags:
+            for k in parsed_args.tags:
+                try:
+                    image_client.image_tags.delete(image.id, k)
+                except Exception:
+                    LOG.error(_("tag unset failed, '%s' is a "
+                                "nonexistent tag "), k)
+                    tagret += 1
+
+        if parsed_args.properties:
+            for k in parsed_args.properties:
+                try:
+                    assert(k in image.keys())
+                except AssertionError:
+                    LOG.error(_("property unset failed, '%s' is a "
+                                "nonexistent property "), k)
+                    propret += 1
+            image_client.images.update(
+                image.id,
+                parsed_args.properties,
+                **kwargs)
+
+        tagtotal = len(parsed_args.tags)
+        proptotal = len(parsed_args.properties)
+        if (tagret > 0 and propret > 0):
+            msg = (_("Failed to unset %(tagret)s of %(tagtotal)s tags,"
+                   "Failed to unset %(propret)s of %(proptotal)s properties.")
+                   % {'tagret': tagret, 'tagtotal': tagtotal,
+                      'propret': propret, 'proptotal': proptotal})
+            raise exceptions.CommandError(msg)
+        elif tagret > 0:
+            msg = (_("Failed to unset %(tagret)s of %(tagtotal)s tags.")
+                   % {'tagret': tagret, 'tagtotal': tagtotal})
+            raise exceptions.CommandError(msg)
+        elif propret > 0:
+            msg = (_("Failed to unset %(propret)s of %(proptotal)s"
+                   " properties.")
+                   % {'propret': propret, 'proptotal': proptotal})
+            raise exceptions.CommandError(msg)
