@@ -42,6 +42,7 @@ def _get_columns(item):
     columns = list(item.keys())
     if 'tenant_id' in columns:
         columns.remove('tenant_id')
+    if 'project_id' not in columns:
         columns.append('project_id')
     return tuple(sorted(columns))
 
@@ -155,7 +156,7 @@ def _get_attrs_compute(client_manager, parsed_args):
 
 
 class CreateNetwork(common.NetworkAndComputeShowOne):
-    """Create new network"""
+    _description = _("Create new network")
 
     def update_parser_common(self, parser):
         parser.add_argument(
@@ -275,7 +276,7 @@ class CreateNetwork(common.NetworkAndComputeShowOne):
 
 
 class DeleteNetwork(common.NetworkAndComputeDelete):
-    """Delete network(s)"""
+    _description = _("Delete network(s)")
 
     # Used by base class to find resources in parsed_args.
     resource = 'network'
@@ -301,9 +302,9 @@ class DeleteNetwork(common.NetworkAndComputeDelete):
 
 
 class ListNetwork(common.NetworkAndComputeLister):
-    """List networks"""
+    _description = _("List networks")
 
-    def update_parser_common(self, parser):
+    def update_parser_network(self, parser):
         router_ext_group = parser.add_mutually_exclusive_group()
         router_ext_group.add_argument(
             '--external',
@@ -360,6 +361,29 @@ class ListNetwork(common.NetworkAndComputeLister):
             help=_("List networks according to their status "
                    "('ACTIVE', 'BUILD', 'DOWN', 'ERROR')")
         )
+        parser.add_argument(
+            '--provider-network-type',
+            metavar='<provider-network-type>',
+            choices=['flat', 'geneve', 'gre', 'local',
+                     'vlan', 'vxlan'],
+            help=_("List networks according to their physical mechanisms. "
+                   "The supported options are: flat, geneve, gre, local, "
+                   "vlan, vxlan.")
+        )
+        parser.add_argument(
+            '--provider-physical-network',
+            metavar='<provider-physical-network>',
+            dest='physical_network',
+            help=_("List networks according to name of the physical network")
+        )
+        parser.add_argument(
+            '--provider-segment',
+            metavar='<provider-segment>',
+            dest='segmentation_id',
+            help=_("List networks according to VLAN ID for VLAN networks "
+                   "or Tunnel ID for GENEVE/GRE/VXLAN networks")
+        )
+
         return parser
 
     def take_action_network(self, client, parsed_args):
@@ -432,6 +456,13 @@ class ListNetwork(common.NetworkAndComputeLister):
         if parsed_args.status:
             args['status'] = parsed_args.status
 
+        if parsed_args.provider_network_type:
+            args['provider:network_type'] = parsed_args.provider_network_type
+        if parsed_args.physical_network:
+            args['provider:physical_network'] = parsed_args.physical_network
+        if parsed_args.segmentation_id:
+            args['provider:segmentation_id'] = parsed_args.segmentation_id
+
         data = client.networks(**args)
 
         return (column_headers,
@@ -462,7 +493,7 @@ class ListNetwork(common.NetworkAndComputeLister):
 
 
 class SetNetwork(command.Command):
-    """Set network properties"""
+    _description = _("Set network properties")
 
     def get_parser(self, prog_name):
         parser = super(SetNetwork, self).get_parser(prog_name)
@@ -553,7 +584,7 @@ class SetNetwork(command.Command):
 
 
 class ShowNetwork(common.NetworkAndComputeShowOne):
-    """Show network details"""
+    _description = _("Show network details")
 
     def update_parser_common(self, parser):
         parser.add_argument(
