@@ -109,6 +109,8 @@ def _get_attrs(client_manager, parsed_args):
             'The --host-id option is deprecated, '
             'please use --host instead.'
         ))
+    if parsed_args.description is not None:
+        attrs['description'] = parsed_args.description
     if parsed_args.fixed_ip is not None:
         attrs['fixed_ips'] = parsed_args.fixed_ip
     if parsed_args.device:
@@ -180,50 +182,55 @@ def _prepare_fixed_ips(client_manager, parsed_args):
 
 
 def _add_updatable_args(parser):
-        # NOTE(dtroyer): --device-id is deprecated in Mar 2016.  Do not
-        #                remove before 3.x release or Mar 2017.
-        device_group = parser.add_mutually_exclusive_group()
-        device_group.add_argument(
-            '--device',
-            metavar='<device-id>',
-            help=_("Port device ID")
-        )
-        device_group.add_argument(
-            '--device-id',
-            metavar='<device-id>',
-            help=argparse.SUPPRESS,
-        )
-        parser.add_argument(
-            '--device-owner',
-            metavar='<device-owner>',
-            help=_("Device owner of this port. This is the entity that uses "
-                   "the port (for example, network:dhcp).")
-        )
-        parser.add_argument(
-            '--vnic-type',
-            metavar='<vnic-type>',
-            choices=['direct', 'direct-physical', 'macvtap',
-                     'normal', 'baremetal'],
-            help=_("VNIC type for this port (direct | direct-physical | "
-                   "macvtap | normal | baremetal, default: normal)")
-        )
-        # NOTE(dtroyer): --host-id is deprecated in Mar 2016.  Do not
-        #                remove before 3.x release or Mar 2017.
-        host_group = parser.add_mutually_exclusive_group()
-        host_group.add_argument(
-            '--host',
-            metavar='<host-id>',
-            help=_("Allocate port on host <host-id> (ID only)")
-        )
-        host_group.add_argument(
-            '--host-id',
-            metavar='<host-id>',
-            help=argparse.SUPPRESS,
-        )
+    parser.add_argument(
+        '--description',
+        metavar='<description>',
+        help=_("Description of this port")
+    )
+    # NOTE(dtroyer): --device-id is deprecated in Mar 2016.  Do not
+    #                remove before 3.x release or Mar 2017.
+    device_group = parser.add_mutually_exclusive_group()
+    device_group.add_argument(
+        '--device',
+        metavar='<device-id>',
+        help=_("Port device ID")
+    )
+    device_group.add_argument(
+        '--device-id',
+        metavar='<device-id>',
+        help=argparse.SUPPRESS,
+    )
+    parser.add_argument(
+        '--device-owner',
+        metavar='<device-owner>',
+        help=_("Device owner of this port. This is the entity that uses "
+               "the port (for example, network:dhcp).")
+    )
+    parser.add_argument(
+        '--vnic-type',
+        metavar='<vnic-type>',
+        choices=['direct', 'direct-physical', 'macvtap',
+                 'normal', 'baremetal'],
+        help=_("VNIC type for this port (direct | direct-physical | "
+               "macvtap | normal | baremetal, default: normal)")
+    )
+    # NOTE(dtroyer): --host-id is deprecated in Mar 2016.  Do not
+    #                remove before 3.x release or Mar 2017.
+    host_group = parser.add_mutually_exclusive_group()
+    host_group.add_argument(
+        '--host',
+        metavar='<host-id>',
+        help=_("Allocate port on host <host-id> (ID only)")
+    )
+    host_group.add_argument(
+        '--host-id',
+        metavar='<host-id>',
+        help=argparse.SUPPRESS,
+    )
 
 
 class CreatePort(command.ShowOne):
-    """Create a new port"""
+    _description = _("Create a new port")
 
     def get_parser(self, prog_name):
         parser = super(CreatePort, self).get_parser(prog_name)
@@ -281,7 +288,23 @@ class CreatePort(command.ShowOne):
             help=_("Name of this port")
         )
         # TODO(singhj): Add support for extended options:
-        # qos,security groups,dhcp, address pairs
+        # qos,dhcp, address pairs
+        secgroups = parser.add_mutually_exclusive_group()
+        secgroups.add_argument(
+            '--security-group',
+            metavar='<security-group>',
+            action='append',
+            dest='security_groups',
+            help=_("Security group to associate with this port (name or ID) "
+                   "(repeat option to set multiple security groups)")
+        )
+        secgroups.add_argument(
+            '--no-security-group',
+            dest='no_security_group',
+            action='store_true',
+            help=_("Associate no security groups with this port")
+        )
+
         return parser
 
     def take_action(self, parsed_args):
@@ -291,6 +314,14 @@ class CreatePort(command.ShowOne):
         parsed_args.network = _network.id
         _prepare_fixed_ips(self.app.client_manager, parsed_args)
         attrs = _get_attrs(self.app.client_manager, parsed_args)
+
+        if parsed_args.security_groups:
+            attrs['security_groups'] = [client.find_security_group(
+                                        sg, ignore_missing=False).id
+                                        for sg in parsed_args.security_groups]
+        if parsed_args.no_security_group:
+            attrs['security_groups'] = []
+
         obj = client.create_port(**attrs)
         columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns, formatters=_formatters)
@@ -299,7 +330,7 @@ class CreatePort(command.ShowOne):
 
 
 class DeletePort(command.Command):
-    """Delete port(s)"""
+    _description = _("Delete port(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeletePort, self).get_parser(prog_name)
@@ -333,7 +364,7 @@ class DeletePort(command.Command):
 
 
 class ListPort(command.Lister):
-    """List ports"""
+    _description = _("List ports")
 
     def get_parser(self, prog_name):
         parser = super(ListPort, self).get_parser(prog_name)
@@ -360,6 +391,17 @@ class ListPort(command.Lister):
             metavar='<server>',
             help=_("List only ports attached to this server (name or ID)"),
         )
+        parser.add_argument(
+            '--mac-address',
+            metavar='<mac-address>',
+            help=_("List only ports with this MAC address")
+        )
+        parser.add_argument(
+            '--long',
+            action='store_true',
+            default=False,
+            help=_("List additional fields in output")
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -371,15 +413,20 @@ class ListPort(command.Lister):
             'name',
             'mac_address',
             'fixed_ips',
+            'status',
         )
         column_headers = (
             'ID',
             'Name',
             'MAC Address',
             'Fixed IP Addresses',
+            'Status',
         )
 
         filters = {}
+        if parsed_args.long:
+            columns += ('security_groups', 'device_owner',)
+            column_headers += ('Security Groups', 'Device Owner',)
         if parsed_args.device_owner is not None:
             filters['device_owner'] = parsed_args.device_owner
         if parsed_args.router:
@@ -394,6 +441,8 @@ class ListPort(command.Lister):
             network = network_client.find_network(parsed_args.network,
                                                   ignore_missing=False)
             filters['network_id'] = network.id
+        if parsed_args.mac_address:
+            filters['mac_address'] = parsed_args.mac_address
 
         data = network_client.ports(**filters)
 
@@ -405,7 +454,7 @@ class ListPort(command.Lister):
 
 
 class SetPort(command.Command):
-    """Set port properties"""
+    _description = _("Set port properties")
 
     def get_parser(self, prog_name):
         parser = super(SetPort, self).get_parser(prog_name)
@@ -463,6 +512,21 @@ class SetPort(command.Command):
             metavar="<port>",
             help=_("Port to modify (name or ID)")
         )
+        parser.add_argument(
+            '--security-group',
+            metavar='<security-group>',
+            action='append',
+            dest='security_groups',
+            help=_("Security group to associate with this port (name or ID) "
+                   "(repeat option to set multiple security groups)")
+        )
+        parser.add_argument(
+            '--no-security-group',
+            dest='no_security_group',
+            action='store_true',
+            help=_("Clear existing security groups associated with this port")
+        )
+
         return parser
 
     def take_action(self, parsed_args):
@@ -490,12 +554,23 @@ class SetPort(command.Command):
                 attrs['fixed_ips'] += [ip for ip in obj.fixed_ips if ip]
         elif parsed_args.no_fixed_ip:
             attrs['fixed_ips'] = []
+        if parsed_args.security_groups and parsed_args.no_security_group:
+            attrs['security_groups'] = [client.find_security_group(sg,
+                                        ignore_missing=False).id
+                                        for sg in parsed_args.security_groups]
+        elif parsed_args.security_groups:
+            attrs['security_groups'] = obj.security_groups
+            for sg in parsed_args.security_groups:
+                sg_id = client.find_security_group(sg, ignore_missing=False).id
+                attrs['security_groups'].append(sg_id)
+        elif parsed_args.no_security_group:
+            attrs['security_groups'] = []
 
         client.update_port(obj, **attrs)
 
 
 class ShowPort(command.ShowOne):
-    """Display port details"""
+    _description = _("Display port details")
 
     def get_parser(self, prog_name):
         parser = super(ShowPort, self).get_parser(prog_name)
@@ -515,7 +590,7 @@ class ShowPort(command.ShowOne):
 
 
 class UnsetPort(command.Command):
-    """Unset port properties"""
+    _description = _("Unset port properties")
 
     def get_parser(self, prog_name):
         parser = super(UnsetPort, self).get_parser(prog_name)
@@ -535,6 +610,15 @@ class UnsetPort(command.Command):
             action='append',
             help=_("Desired key which should be removed from binding:profile"
                    "(repeat option to unset multiple binding:profile data)"))
+        parser.add_argument(
+            '--security-group',
+            metavar='<security-group>',
+            action='append',
+            dest='security_groups',
+            help=_("Security group which should be removed this port (name "
+                   "or ID) (repeat option to unset multiple security groups)")
+        )
+
         parser.add_argument(
             'port',
             metavar="<port>",
@@ -550,6 +634,7 @@ class UnsetPort(command.Command):
         # Unset* classes
         tmp_fixed_ips = copy.deepcopy(obj.fixed_ips)
         tmp_binding_profile = copy.deepcopy(obj.binding_profile)
+        tmp_secgroups = copy.deepcopy(obj.security_groups)
         _prepare_fixed_ips(self.app.client_manager, parsed_args)
         attrs = {}
         if parsed_args.fixed_ip:
@@ -568,5 +653,16 @@ class UnsetPort(command.Command):
                 msg = _("Port does not contain binding-profile %s") % key
                 raise exceptions.CommandError(msg)
             attrs['binding:profile'] = tmp_binding_profile
+        if parsed_args.security_groups:
+            try:
+                for sg in parsed_args.security_groups:
+                    sg_id = client.find_security_group(
+                        sg, ignore_missing=False).id
+                    tmp_secgroups.remove(sg_id)
+            except ValueError:
+                msg = _("Port does not contain security group %s") % sg
+                raise exceptions.CommandError(msg)
+            attrs['security_groups'] = tmp_secgroups
+
         if attrs:
             client.update_port(obj, **attrs)
