@@ -12,16 +12,32 @@
 #
 
 """Subnet action implementations"""
-import copy
 
-from openstackclient.common import command
-from openstackclient.common import exceptions
-from openstackclient.common import parseractions
-from openstackclient.common import utils
+import copy
+import logging
+
+from osc_lib.cli import parseractions
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
+
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
 
 
+LOG = logging.getLogger(__name__)
+
+
+def _update_arguments(obj_list, parsed_args_list, option):
+    for item in parsed_args_list:
+        try:
+            obj_list.remove(item)
+        except ValueError:
+            msg = (_("Subnet does not contain %(option)s %(value)s") %
+                   {'option': option, 'value': item})
+            raise exceptions.CommandError(msg)
+
+
 def _format_allocation_pools(data):
     pool_formatted = ['%s-%s' % (pool.get('start', ''), pool.get('end', ''))
                       for pool in data]
@@ -37,10 +53,11 @@ _formatters = {
     'allocation_pools': _format_allocation_pools,
     'dns_nameservers': utils.format_list,
     'host_routes': _format_host_routes,
+    'service_types': utils.format_list,
 }
 
 
-def _get_common_parse_arguments(parser):
+def _get_common_parse_arguments(parser, is_create=True):
     parser.add_argument(
         '--allocation-pool',
         metavar='start=<ip-address>,end=<ip-address>',
@@ -51,6 +68,14 @@ def _get_common_parse_arguments(parser):
                "e.g.: start=192.168.199.2,end=192.168.199.254 "
                "(repeat option to add multiple IP addresses)")
     )
+    if not is_create:
+        parser.add_argument(
+            '--no-allocation-pool',
+            action='store_true',
+            help=_("Clear associated allocation-pools from the subnet. "
+                   "Specify both --allocation-pool and --no-allocation-pool "
+                   "to overwrite the current allocation pool information.")
+        )
     parser.add_argument(
         '--dns-nameserver',
         metavar='<dns-nameserver>',
@@ -71,6 +96,24 @@ def _get_common_parse_arguments(parser):
                "gateway: nexthop IP address "
                "(repeat option to add multiple routes)")
     )
+    if not is_create:
+        parser.add_argument(
+            '--no-host-route',
+            action='store_true',
+            help=_("Clear associated host-routes from the subnet. "
+                   "Specify both --host-route and --no-host-route "
+                   "to overwrite the current host route information.")
+        )
+    parser.add_argument(
+        '--service-type',
+        metavar='<service-type>',
+        action='append',
+        dest='service_types',
+        help=_("Service type for this subnet "
+               "e.g.: network:floatingip_agent_gateway. "
+               "Must be a valid device owner value for a network port "
+               "(repeat option to set multiple service types)")
+    )
 
 
 def _get_columns(item):
@@ -136,6 +179,9 @@ def _get_attrs(client_manager, parsed_args, is_create=True):
             attrs['ipv6_ra_mode'] = parsed_args.ipv6_ra_mode
         if parsed_args.ipv6_address_mode is not None:
             attrs['ipv6_address_mode'] = parsed_args.ipv6_address_mode
+        if parsed_args.network_segment is not None:
+            attrs['segment_id'] = client.find_segment(
+                parsed_args.network_segment, ignore_missing=False).id
 
     if 'gateway' in parsed_args and parsed_args.gateway is not None:
         gateway = parsed_args.gateway.lower()
@@ -154,7 +200,7 @@ def _get_attrs(client_manager, parsed_args, is_create=True):
         attrs['allocation_pools'] = parsed_args.allocation_pools
     if parsed_args.dhcp:
         attrs['enable_dhcp'] = True
-    elif parsed_args.no_dhcp:
+    if parsed_args.no_dhcp:
         attrs['enable_dhcp'] = False
     if ('dns_nameservers' in parsed_args and
        parsed_args.dns_nameservers is not None):
@@ -163,11 +209,16 @@ def _get_attrs(client_manager, parsed_args, is_create=True):
         # Change 'gateway' entry to 'nexthop' to match the API
         attrs['host_routes'] = convert_entries_to_nexthop(
             parsed_args.host_routes)
+    if ('service_types' in parsed_args and
+       parsed_args.service_types is not None):
+        attrs['service_types'] = parsed_args.service_types
+    if parsed_args.description is not None:
+        attrs['description'] = parsed_args.description
     return attrs
 
 
 class CreateSubnet(command.ShowOne):
-    """Create a subnet"""
+    _description = _("Create a subnet")
 
     def get_parser(self, prog_name):
         parser = super(CreateSubnet, self).get_parser(prog_name)
@@ -209,7 +260,6 @@ class CreateSubnet(command.ShowOne):
         dhcp_enable_group.add_argument(
             '--dhcp',
             action='store_true',
-            default=True,
             help=_("Enable DHCP (default)")
         )
         dhcp_enable_group.add_argument(
@@ -226,7 +276,7 @@ class CreateSubnet(command.ShowOne):
                    "'auto': Gateway address should automatically be chosen "
                    "from within the subnet itself, 'none': This subnet will "
                    "not use a gateway, e.g.: --gateway 192.168.9.1, "
-                   "--gateway auto, --gateway none (default is 'auto')")
+                   "--gateway auto, --gateway none (default is 'auto').")
         )
         parser.add_argument(
             '--ip-version',
@@ -235,7 +285,7 @@ class CreateSubnet(command.ShowOne):
             choices=[4, 6],
             help=_("IP version (default is 4).  Note that when subnet pool is "
                    "specified, IP version is determined from the subnet pool "
-                   "and this option is ignored")
+                   "and this option is ignored.")
         )
         parser.add_argument(
             '--ipv6-ra-mode',
@@ -249,12 +299,23 @@ class CreateSubnet(command.ShowOne):
             help=_("IPv6 address mode, "
                    "valid modes: [dhcpv6-stateful, dhcpv6-stateless, slaac]")
         )
+        parser.add_argument(
+            '--network-segment',
+            metavar='<network-segment>',
+            help=_("Network segment to associate with this subnet "
+                   "(name or ID)")
+        )
         parser.add_argument(
             '--network',
             required=True,
             metavar='<network>',
             help=_("Network this subnet belongs to (name or ID)")
         )
+        parser.add_argument(
+            '--description',
+            metavar='<description>',
+            help=_("Set subnet description")
+        )
         _get_common_parse_arguments(parser)
         return parser
 
@@ -268,25 +329,41 @@ class CreateSubnet(command.ShowOne):
 
 
 class DeleteSubnet(command.Command):
-    """Delete subnet"""
+    _description = _("Delete subnet(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteSubnet, self).get_parser(prog_name)
         parser.add_argument(
             'subnet',
             metavar="<subnet>",
-            help=_("Subnet to delete (name or ID)")
+            nargs='+',
+            help=_("Subnet(s) to delete (name or ID)")
         )
         return parser
 
     def take_action(self, parsed_args):
         client = self.app.client_manager.network
-        client.delete_subnet(
-            client.find_subnet(parsed_args.subnet))
+        result = 0
+
+        for subnet in parsed_args.subnet:
+            try:
+                obj = client.find_subnet(subnet, ignore_missing=False)
+                client.delete_subnet(obj)
+            except Exception as e:
+                result += 1
+                LOG.error(_("Failed to delete subnet with "
+                          "name or ID '%(subnet)s': %(e)s")
+                          % {'subnet': subnet, 'e': e})
+
+        if result > 0:
+            total = len(parsed_args.subnet)
+            msg = (_("%(result)s of %(total)s subnets failed "
+                   "to delete.") % {'result': result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListSubnet(command.Lister):
-    """List subnets"""
+    _description = _("List subnets")
 
     def get_parser(self, prog_name):
         parser = super(ListSubnet, self).get_parser(prog_name)
@@ -302,26 +379,102 @@ class ListSubnet(command.Lister):
             choices=[4, 6],
             metavar='<ip-version>',
             dest='ip_version',
-            help=_("List only subnets of given IP version in output"
+            help=_("List only subnets of given IP version in output."
                    "Allowed values for IP version are 4 and 6."),
         )
+        dhcp_enable_group = parser.add_mutually_exclusive_group()
+        dhcp_enable_group.add_argument(
+            '--dhcp',
+            action='store_true',
+            help=_("List subnets which have DHCP enabled")
+        )
+        dhcp_enable_group.add_argument(
+            '--no-dhcp',
+            action='store_true',
+            help=_("List subnets which have DHCP disabled")
+        )
+        parser.add_argument(
+            '--service-type',
+            metavar='<service-type>',
+            action='append',
+            dest='service_types',
+            help=_("List only subnets of a given service type in output "
+                   "e.g.: network:floatingip_agent_gateway. "
+                   "Must be a valid device owner value for a network port "
+                   "(repeat option to list multiple service types)")
+        )
+        parser.add_argument(
+            '--project',
+            metavar='<project>',
+            help=_("List only subnets which belong to a given project "
+                   "(name or ID) in output")
+        )
+        identity_common.add_project_domain_option_to_parser(parser)
+        parser.add_argument(
+            '--network',
+            metavar='<network>',
+            help=_("List only subnets which belong to a given network "
+                   "(name or ID) in output")
+        )
+        parser.add_argument(
+            '--gateway',
+            metavar='<gateway>',
+            help=_("List only subnets of given gateway IP in output")
+        )
+        parser.add_argument(
+            '--name',
+            metavar='<name>',
+            help=_("List only subnets of given name in output")
+        )
+        parser.add_argument(
+            '--subnet-range',
+            metavar='<subnet-range>',
+            help=_("List only subnets of given subnet range "
+                   "(in CIDR notation) in output "
+                   "e.g.: --subnet-range 10.10.0.0/16")
+        )
         return parser
 
     def take_action(self, parsed_args):
+        identity_client = self.app.client_manager.identity
+        network_client = self.app.client_manager.network
         filters = {}
         if parsed_args.ip_version:
             filters['ip_version'] = parsed_args.ip_version
-        data = self.app.client_manager.network.subnets(**filters)
+        if parsed_args.dhcp:
+            filters['enable_dhcp'] = True
+        elif parsed_args.no_dhcp:
+            filters['enable_dhcp'] = False
+        if parsed_args.service_types:
+            filters['service_types'] = parsed_args.service_types
+        if parsed_args.project:
+            project_id = identity_common.find_project(
+                identity_client,
+                parsed_args.project,
+                parsed_args.project_domain,
+            ).id
+            filters['tenant_id'] = project_id
+        if parsed_args.network:
+            network_id = network_client.find_network(parsed_args.network,
+                                                     ignore_missing=False).id
+            filters['network_id'] = network_id
+        if parsed_args.gateway:
+            filters['gateway_ip'] = parsed_args.gateway
+        if parsed_args.name:
+            filters['name'] = parsed_args.name
+        if parsed_args.subnet_range:
+            filters['cidr'] = parsed_args.subnet_range
+        data = network_client.subnets(**filters)
 
         headers = ('ID', 'Name', 'Network', 'Subnet')
         columns = ('id', 'name', 'network_id', 'cidr')
         if parsed_args.long:
             headers += ('Project', 'DHCP', 'Name Servers',
                         'Allocation Pools', 'Host Routes', 'IP Version',
-                        'Gateway')
+                        'Gateway', 'Service Types')
             columns += ('tenant_id', 'enable_dhcp', 'dns_nameservers',
                         'allocation_pools', 'host_routes', 'ip_version',
-                        'gateway_ip')
+                        'gateway_ip', 'service_types')
 
         return (headers,
                 (utils.get_item_properties(
@@ -331,7 +484,7 @@ class ListSubnet(command.Lister):
 
 
 class SetSubnet(command.Command):
-    """Set subnet properties"""
+    _description = _("Set subnet properties")
 
     def get_parser(self, prog_name):
         parser = super(SetSubnet, self).get_parser(prog_name)
@@ -363,9 +516,14 @@ class SetSubnet(command.Command):
             help=_("Specify a gateway for the subnet. The options are: "
                    "<ip-address>: Specific IP address to use as the gateway, "
                    "'none': This subnet will not use a gateway, "
-                   "e.g.: --gateway 192.168.9.1, --gateway none")
+                   "e.g.: --gateway 192.168.9.1, --gateway none.")
         )
-        _get_common_parse_arguments(parser)
+        parser.add_argument(
+            '--description',
+            metavar='<description>',
+            help=_("Set subnet description")
+        )
+        _get_common_parse_arguments(parser, is_create=False)
         return parser
 
     def take_action(self, parsed_args):
@@ -373,21 +531,26 @@ class SetSubnet(command.Command):
         obj = client.find_subnet(parsed_args.subnet, ignore_missing=False)
         attrs = _get_attrs(self.app.client_manager, parsed_args,
                            is_create=False)
-        if not attrs:
-            msg = "Nothing specified to be set"
-            raise exceptions.CommandError(msg)
         if 'dns_nameservers' in attrs:
             attrs['dns_nameservers'] += obj.dns_nameservers
         if 'host_routes' in attrs:
-            attrs['host_routes'] += obj.host_routes
+            if not parsed_args.no_host_route:
+                attrs['host_routes'] += obj.host_routes
+        elif parsed_args.no_host_route:
+            attrs['host_routes'] = ''
         if 'allocation_pools' in attrs:
-            attrs['allocation_pools'] += obj.allocation_pools
+            if not parsed_args.no_allocation_pool:
+                attrs['allocation_pools'] += obj.allocation_pools
+        elif parsed_args.no_allocation_pool:
+            attrs['allocation_pools'] = []
+        if 'service_types' in attrs:
+            attrs['service_types'] += obj.service_types
         client.update_subnet(obj, **attrs)
         return
 
 
 class ShowSubnet(command.ShowOne):
-    """Display subnet details"""
+    _description = _("Display subnet details")
 
     def get_parser(self, prog_name):
         parser = super(ShowSubnet, self).get_parser(prog_name)
@@ -404,3 +567,85 @@ class ShowSubnet(command.ShowOne):
         columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns, formatters=_formatters)
         return (columns, data)
+
+
+class UnsetSubnet(command.Command):
+    _description = _("Unset subnet properties")
+
+    def get_parser(self, prog_name):
+        parser = super(UnsetSubnet, self).get_parser(prog_name)
+        parser.add_argument(
+            '--allocation-pool',
+            metavar='start=<ip-address>,end=<ip-address>',
+            dest='allocation_pools',
+            action=parseractions.MultiKeyValueAction,
+            required_keys=['start', 'end'],
+            help=_('Allocation pool IP addresses to be removed from this '
+                   'subnet e.g.: start=192.168.199.2,end=192.168.199.254 '
+                   '(repeat option to unset multiple allocation pools)')
+        )
+        parser.add_argument(
+            '--dns-nameserver',
+            metavar='<dns-nameserver>',
+            action='append',
+            dest='dns_nameservers',
+            help=_('DNS server to be removed from this subnet '
+                   '(repeat option to unset multiple DNS servers)')
+        )
+        parser.add_argument(
+            '--host-route',
+            metavar='destination=<subnet>,gateway=<ip-address>',
+            dest='host_routes',
+            action=parseractions.MultiKeyValueAction,
+            required_keys=['destination', 'gateway'],
+            help=_('Route to be removed from this subnet '
+                   'e.g.: destination=10.10.0.0/16,gateway=192.168.71.254 '
+                   'destination: destination subnet (in CIDR notation) '
+                   'gateway: nexthop IP address '
+                   '(repeat option to unset multiple host routes)')
+        )
+        parser.add_argument(
+            '--service-type',
+            metavar='<service-type>',
+            action='append',
+            dest='service_types',
+            help=_('Service type to be removed from this subnet '
+                   'e.g.: network:floatingip_agent_gateway. '
+                   'Must be a valid device owner value for a network port '
+                   '(repeat option to unset multiple service types)')
+        )
+        parser.add_argument(
+            'subnet',
+            metavar="<subnet>",
+            help=_("Subnet to modify (name or ID)")
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        client = self.app.client_manager.network
+        obj = client.find_subnet(parsed_args.subnet, ignore_missing=False)
+        tmp_obj = copy.deepcopy(obj)
+        attrs = {}
+        if parsed_args.dns_nameservers:
+            _update_arguments(tmp_obj.dns_nameservers,
+                              parsed_args.dns_nameservers,
+                              'dns-nameserver')
+            attrs['dns_nameservers'] = tmp_obj.dns_nameservers
+        if parsed_args.host_routes:
+            _update_arguments(
+                tmp_obj.host_routes,
+                convert_entries_to_nexthop(parsed_args.host_routes),
+                'host-route')
+            attrs['host_routes'] = tmp_obj.host_routes
+        if parsed_args.allocation_pools:
+            _update_arguments(tmp_obj.allocation_pools,
+                              parsed_args.allocation_pools,
+                              'allocation-pool')
+            attrs['allocation_pools'] = tmp_obj.allocation_pools
+        if parsed_args.service_types:
+            _update_arguments(tmp_obj.service_types,
+                              parsed_args.service_types,
+                              'service-type')
+            attrs['service_types'] = tmp_obj.service_types
+        if attrs:
+            client.update_subnet(obj, **attrs)
