@@ -996,6 +996,96 @@ class TestVolumeList(TestVolume):
                           self.cmd, arglist, verifylist)
 
 
+class TestVolumeMigrate(TestVolume):
+
+    _volume = volume_fakes.FakeVolume.create_one_volume()
+
+    def setUp(self):
+        super(TestVolumeMigrate, self).setUp()
+
+        self.volumes_mock.get.return_value = self._volume
+        self.volumes_mock.migrate_volume.return_value = None
+        # Get the command object to test
+        self.cmd = volume.MigrateVolume(self.app, None)
+
+    def test_volume_migrate(self):
+        arglist = [
+            "--host", "host@backend-name#pool",
+            self._volume.id,
+        ]
+        verifylist = [
+            ("force_host_copy", False),
+            ("lock_volume", False),
+            ("unlock_volume", False),
+            ("host", "host@backend-name#pool"),
+            ("volume", self._volume.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.volumes_mock.get.assert_called_once_with(self._volume.id)
+        self.volumes_mock.migrate_volume.assert_called_once_with(
+            self._volume.id, "host@backend-name#pool", False, False)
+        self.assertIsNone(result)
+
+    def test_volume_migrate_with_option(self):
+        arglist = [
+            "--force-host-copy",
+            "--lock-volume",
+            "--host", "host@backend-name#pool",
+            self._volume.id,
+        ]
+        verifylist = [
+            ("force_host_copy", True),
+            ("lock_volume", True),
+            ("unlock_volume", False),
+            ("host", "host@backend-name#pool"),
+            ("volume", self._volume.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.volumes_mock.get.assert_called_once_with(self._volume.id)
+        self.volumes_mock.migrate_volume.assert_called_once_with(
+            self._volume.id, "host@backend-name#pool", True, True)
+        self.assertIsNone(result)
+
+    def test_volume_migrate_with_unlock_volume(self):
+        arglist = [
+            "--unlock-volume",
+            "--host", "host@backend-name#pool",
+            self._volume.id,
+        ]
+        verifylist = [
+            ("force_host_copy", False),
+            ("lock_volume", False),
+            ("unlock_volume", True),
+            ("host", "host@backend-name#pool"),
+            ("volume", self._volume.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.volumes_mock.get.assert_called_once_with(self._volume.id)
+        self.volumes_mock.migrate_volume.assert_called_once_with(
+            self._volume.id, "host@backend-name#pool", False, False)
+        self.assertIsNone(result)
+
+    def test_volume_migrate_without_host(self):
+        arglist = [
+            self._volume.id,
+        ]
+        verifylist = [
+            ("force_host_copy", False),
+            ("lock_volume", False),
+            ("unlock_volume", False),
+            ("volume", self._volume.id),
+        ]
+
+        self.assertRaises(tests_utils.ParserException, self.check_parser,
+                          self.cmd, arglist, verifylist)
+
+
 class TestVolumeSet(TestVolume):
 
     def setUp(self):
@@ -1033,6 +1123,8 @@ class TestVolumeSet(TestVolume):
             self.new_volume.id
         ]
         verifylist = [
+            ('read_only', False),
+            ('read_write', False),
             ('state', 'error'),
             ('volume', self.new_volume.id)
         ]
@@ -1042,6 +1134,7 @@ class TestVolumeSet(TestVolume):
         result = self.cmd.take_action(parsed_args)
         self.volumes_mock.reset_state.assert_called_with(
             self.new_volume.id, 'error')
+        self.volumes_mock.update_readonly_flag.assert_not_called()
         self.assertIsNone(result)
 
     def test_volume_set_state_failed(self):
@@ -1090,6 +1183,44 @@ class TestVolumeSet(TestVolume):
             self.volumes_mock.set_bootable.assert_called_with(
                 self.new_volume.id, verifylist[index][0][1])
 
+    def test_volume_set_readonly(self):
+        arglist = [
+            '--read-only',
+            self.new_volume.id
+        ]
+        verifylist = [
+            ('read_only', True),
+            ('read_write', False),
+            ('volume', self.new_volume.id)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.volumes_mock.update_readonly_flag.assert_called_once_with(
+            self.new_volume.id,
+            True)
+        self.assertIsNone(result)
+
+    def test_volume_set_read_write(self):
+        arglist = [
+            '--read-write',
+            self.new_volume.id
+        ]
+        verifylist = [
+            ('read_only', False),
+            ('read_write', True),
+            ('volume', self.new_volume.id)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.volumes_mock.update_readonly_flag.assert_called_once_with(
+            self.new_volume.id,
+            False)
+        self.assertIsNone(result)
+
 
 class TestVolumeShow(TestVolume):
 
