@@ -30,7 +30,7 @@ LOG = logging.getLogger(__name__)
 
 
 class CreateVolumeType(command.ShowOne):
-    """Create new volume type"""
+    _description = _("Create new volume type")
 
     def get_parser(self, prog_name):
         parser = super(CreateVolumeType, self).get_parser(prog_name)
@@ -116,7 +116,7 @@ class CreateVolumeType(command.ShowOne):
 
 
 class DeleteVolumeType(command.Command):
-    """Delete volume type(s)"""
+    _description = _("Delete volume type(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteVolumeType, self).get_parser(prog_name)
@@ -152,7 +152,7 @@ class DeleteVolumeType(command.Command):
 
 
 class ListVolumeType(command.Lister):
-    """List volume types"""
+    _description = _("List volume types")
 
     def get_parser(self, prog_name):
         parser = super(ListVolumeType, self).get_parser(prog_name)
@@ -197,7 +197,7 @@ class ListVolumeType(command.Lister):
 
 
 class SetVolumeType(command.Command):
-    """Set volume type properties"""
+    _description = _("Set volume type properties")
 
     def get_parser(self, prog_name):
         parser = super(SetVolumeType, self).get_parser(prog_name)
@@ -286,7 +286,7 @@ class SetVolumeType(command.Command):
 
 
 class ShowVolumeType(command.ShowOne):
-    """Display volume type details"""
+    _description = _("Display volume type details")
 
     def get_parser(self, prog_name):
         parser = super(ShowVolumeType, self).get_parser(prog_name)
@@ -324,7 +324,7 @@ class ShowVolumeType(command.ShowOne):
 
 
 class UnsetVolumeType(command.Command):
-    """Unset volume type properties"""
+    _description = _("Unset volume type properties")
 
     def get_parser(self, prog_name):
         parser = super(UnsetVolumeType, self).get_parser(prog_name)
